;8 MHZ clock used for this coding
;   Fuses set for 8 MHZ internal clock, brown out detection 1.8 volts
;	internal oscillator at 8 MHz 14CK+65ms[CKSEL=0100 SUT= ],
;	NO divide by 8

; Power Supply about 5 volts

;FUSES:

;Brown-out detection level at VCC=2.7 V;[BODLEVEL=101]
;Int. RC Osc. 8MHz Start-up time 14 CK + 65ms [CKSEL=0100 SUT=10]


;FLAGS -   r18

;r18,0(1) - 1 = relay ON,   2 = relay OFF
;r18,1(2) - Beep Out Data at Intervals  1 = ON
;r18,2(4) - Initial relay status 1= ON
;r18,3(8) - Heat = 1,  Cool = 2 - normal or reversed relay power
;r18,4(16) - 
;r18,5(32) - 1 = temperature HIGHER than reference, 2 = Lower than reference
;r18,6(64) - Sonalert warning (at start of each interval) 1 = ON
;r18,7(128) - Relay Action flag  1 = ON



.include "tn2313def.inc"

	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,91		;Set bits PD0,1,3,4,6 to ones for out, bit PD2,5 for inputs
out		DDRD,r16	;Execute data direction with PD0 as input rest as output

clr		r16			 
out		PORTD,R16	;execute to set out ports low with IN ports 2 and 5 with 
					; no input resistor

sbi		PortD,4		;set Port D4 high (capacitor discharge circuit for counting)
					
ser		R16			;Set all bits for data direction as outputs
out		DDRB,R16	;execute data direction with all as outputs
clr		r16		;
out		PORTB,R16	;execute to set all out ports low


clr		r18			;clear FLAGS register, R18
sbr		r18,72		;set r18,3(8) flag to warm when relay closes - Sonalert On

rcall	iOneBeep		;Beep Sonalert twice upon power up


;---------------------------

MainLoop:

rcall	iDelay1min		;Delay to wait for new temp sample (interruptible)

sbis	PIND,2			;skip if switch is up
rjmp	MLButtonPush

rcall	iCheckTemp		;call the series of programs to check temp, do a stack save (for two 
						;  iterations, make a comparison with reference switching temp, then
						;  display results.  This happens once each minute.

ML2:

rjmp	MainLoop


MLButtonPush:

rcall 	ButtonPush
rcall	Decode
rjmp	ML2


;----------------------------------

iCheckTemp:				;call the series of programs to check temp, do a stack save (for two 
						;  iterations, make a comparison with reference switching temp, then
						;  display results.  This happens once each minute.

clr		r28
clr		r29

cbi		PortD,4


ML3a:

adiw	r28,1			;increment r28,29 "Y" pair by one to count delay of charging capacitor
brcs	ML3aa
sbic	PIND,5			;Start counting while capacitor is discharging - (off PIND,5 port)

rjmp	ML3a			;jump back if capacitor not discharged

sbi		PortD,4			;turn Port D4 back on after capacitor discharge time check)
rjmp	ML3aaa


ML3aa:					;a jump to here if overflow occurs (for testing purposes in programmer)
						;Set up 1234 display IF there is an overflow (mainly for test purposes)

sbr		r18,2				;set beep out data flag ON
ldi		r28,210				;load 1234 into r28,29
ldi		r29,4


;count in r28,29 now complete --- next, do a compare to reference


ML3aaa:

sts		$64,r28
sts		$65,r29			;store temperature count in SRAM

;Start comparison against reference temperature in $60,$61

lds		r24,$60
lds		r25,$61			;load switching reference into r24,25 "W" pair

cp		r25,r29			;check HIGH byte
breq	ML3e			;check low byte next
brcs	ML3b			;branch if count is higher than reference


ML3c:					;count is same or lower than reference switching number

cbr		r18,32			;clear r18,5(32) (count is lower than reference)
rjmp	ML3end


ML3b:					;count is higher than reference

sbr		r18,32			;set r18,5(32) (count is higher than reference)
rjmp	ML3end


ML3e:					;check lower byte, r28

cp		r24,r28
brcs	ML3b			;count is higher than reference
rjmp	ML3c			;count is same or lower than reference switching count

;(count is lower or same as reference)

ML3end:					;comparison complete


;Two consecutive reads must be the same (relay on or off) if relay is to actually be switched

MLstack:					;Part of Main Loop (ML series)

lds		r16,$69				;stack = $69 (using two LS bits for two consecutive temp readings
							;  are consecutively consistent

lsl		r16					;shift old, previous bit one left

sbrs	R18,5				;skip if r18,5(32) bit is set (freq > ref)
rjmp	MLstack1a			;  freq is lower than ref

sbr		r16,1				;set r16,0(1) (freq > ref)

rjmp	MLstack1b


MLstack1a:					;(freq < ref)

cbr		r16,1


MLstack1b:

sts		$69,r16				;store away r16 back into $69 SRAM


MLstackC:					;isolate two LS bits in r16

andi	r16,3				;and 3 to isolate bits

cpi		r16,3	
breq	MLstack4

cpi		r16,0
breq	MLstack5

rjmp	MLrelayEnd			;take no action -jump past rest and go directly to 
							;  MLrelayEnd


MLstack4:

sbr		r18,128				;set r18,7(128) bit (relay action flag)
rjmp	MLstackEnd


MLstack5:

cbr		r18,128				;clr r18,7(128) bit (relay action flag)


MLstackEnd:					;END of process (r18,7 now has relay action set to either
							;	1= on relay or 0= off relay

;Following reverses relay action if flag is not set


MLrelay4:					;RELAY ACTION  ---	process relay action flag

sbrc	r18,3				;skip if r18,3(8) is clear (reverse relay action if clear)
rjmp	MLrelay5			;normal relay action if set


MLrelay4a:					;REVERSE relay action

sbrs	r18,7				;skip if relay action r18,7(128) is set
rjmp	MLrelay4a2			;relay action flag NOT set
cbr		r18,128				;reverse relay action
rjmp	MLrelay5			;to continue


MLrelay4a2:

sbr		r18,128
rjmp	MLrelay5			;to continue (reversal of relay action complete)

;At this point forward, relay is turned on or off with following code


MLrelay5:

sbrs	r18,7			;skip if r18,7(128) relay action flag is set -- 1 = relay ON
rjmp	MLrelay6		; r18,7(128) NOT set -- relay OFF


MLrelay7:				;ON relay

sbrc	r18,0			;skip if r18,0(1) is clear -- 0 = OFF
rjmp	MLrelayEnd
rcall	PulseRelayON
rjmp	MLrelayEnd


MLrelay6:				;OFF relay

sbrs	r18,0
rjmp	MLrelayEnd

rcall	PulseRelayOFF
rjmp	MLrelayEnd


MLrelayEnd:

rcall	iFinalDisplay

ret						;return back into Main Loop


;----------------------------

StallButtonPush:	;stalls and waits for button push entry.

sei	;ON interrupts to keep timing during button push routines
	;using crystal clock


SBP1:

sbic	PIND,2	;skip if PIND,2, switch is down
rjmp	SBP1

rjmp	ButtonPush


;-----------------------------------------------------------

;                 r30,31 has complete count.
;                 r21 has one less count of LSB value in r30 -- used for plus one entries


ButtonPush:  ;Takes in dit, dah, holddown, and coded input

push	r0
push	r1
push	r2
push	r3
push	r4
push	r7
push	r8
push	r9
push	r21
push	r23
push	r24
push	r25

cbr		r19,1  ;clear r19,0(1), kill delay flag -- turn it off


;r30,r31 (Z register pair contain total value of entry)

clr		r0	;clear number of bits in code
clr		r1	;clear lowest code count
clr		r2	;clear next code count
clr		r3	;clear next code count
clr		r4	;clear next code count (clear all code counts)
clr		r7	;HoldDown count
clr		r8	;dah count
clr		r9	;Dit Count
clr		r21	;one less count
clr		r23	;universal counter
clr		r24
clr		r25

	;r30,31 has total count

B1:

inc		r23	;universal counter
rcall	Delay025	;1/40th second delay
cpi		r23,60	;delay before first holddown count - 40=one second 
brsh	B3		;branch if - it is a holddown count

sbis	PIND,2		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20	;compare r23 for 20 (1/2 second dah count)
brlo	B2b	;branch if lower (dit count)


B2a:	;it is dah count

inc		r8   	;increment dah count

sec		;set carry flag -- it is dah entry
rol		r1
rol		r2   ;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0		;on red LED
rcall	Delay100	;.1 second delay
cbi		PortB,0		;off red LED
clr		r23
inc		r0    ;increment number of code digits


B2d:	;no touch/switch up detection

sbis	PIND,2	;skip if switch is up
rjmp	B2e	;jump back - there is a new entry coming in

inc		r23
rcall	Delay025	;1/40th second delay
cpi		r23,70	;has switch been up 70 counts? 40=one second (1.75 		;  seconds)
brsh	B4Decode1
rjmp	B2d


B2e:

clr		r23
rjmp	B1


B2b:	;Dit count detected

inc		r9	;increment dit count

clc		;clear carry flag -- it is dit entry
rol		r1
rol		r2	;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0	;On Red LED
rcall	Delay025
cbi		PortB,0	;Off Red LED

inc		r0	;increment number of code digits (r0)
clr		r23	;clear r23
rjmp	B2d


B3:	;Holddown count has been detected

inc		r7  	;increment holddown count
sbi		PortB,0	;On Red LED
rcall	Delay100
cbi		PortB,0	;Off Red LED for short blink

clr		r23	;reset r23 to zero count


B4:

inc		r23
rcall	Delay025
sbic	PIND,2	;skip if switch is down
rjmp	B4a	;jump out and look for next entry

cpi		r23,30	;pause between continuous hold down of switch
brlo	B4

rjmp	B3


B4a:

clr		r23
rjmp	B2d	;Check for exit or new entry



B4Decode1:

;This section finalizes.  It combines counts

clr		r30
clr		r31	;ready registers for input

mov		r16,r8	;move dah count in r8 into r16


B4comb1:

tst		r16
breq	B4comb2

adiw 	r30,10	;add value of 10 in r30,31
dec		r16			
rjmp	B4comb1


B4comb2:

add		r30,r9	;add value in r9 (dit count) to r30
brcs	B4comb2a
rjmp	Bholdown1


B4comb2a:

inc		r31  ;carry was set so increment higher byte


Bholdown1:

;Combined value now in Z register pair, r30,31

mov		r16,r7	;put hold down count into r16

Bholdown1b:

tst		r16		;test r7(r16), hold down count
breq	BPEnd
adiw	r30,50	;add fifty count to r30,31 word total
dec		r16
rjmp	Bholdown1b

;Complete combined total now in Z register pair


BPEnd:

;Following adds 10 to hold down count for W register pair, r24,25

mov		r16,r7	;move number of hold down counts into r16
mov		r24,r30
mov		r25,r31	;transfer total values in r30,31 pair to r24,25 pair


B60a:

tst		r16			
breq	BPEnd2	;r24,25 pair now has X60 values

adiw	r24,10
dec		r16
rjmp	B60a


BPEnd2:

;Store away in SRAM $00E7-00EB, Dit,Dah,HolddownCount,LSD Total, MSD Total

sts		$0070,r0	;number of code digits
sts		$0071,r1	;low byte of code digit
sts		$0072,r2	;next byte of code digit
sts		$0073,r3	;high byte of code digit

sts		$0074,r9	;number of dits
sts		$0075,r8	;number of dahs
sts		$0076,r7	;number of hold down counts
sts		$0077,r30	;LSD of total count
sts		$0078,r31	;MSD of total count 

sts		$0079,r24
sts		$007A,r25	;store X60 total count

mov		r21,r30
dec		r21			;one less count in r30, LSD of total 		;count for Plus One Entries.
sts		$7B,r21		;store one less LS byte in SRAM $7B

 ;r21 has one less count

pop		r25
pop		r24
pop		r23
pop		r21
pop		r9
pop		r8
pop		r7
pop		r4
pop		r3
pop		r2
pop		r1
pop		r0

sbr		r19,1	;set r19,0(1), kill delay flag back on

ret


;---

Decode:

lds		r16,$0076	;get hold down count number


Decode1:			;Immediately flash out current temp

cpi		r16,1
brne	Decode2

rcall	iBeepOutTemp

rjmp	DecodeEnd


Decode2:

cpi		r16,2
brne	Decode3

rcall	ToggleRelayWithEscape		;short beep = ON, long beep = OFF

rjmp 	DecodeEnd


Decode3:							;DISPLAY switch temperature value

cpi		r16,3						;HoldDown 6 flashes
brne	Decode4

rcall	iBeepOutSwitchTemp
rcall	Delay2sec

rjmp	DecodeEnd


Decode4:							;Determines whether relay on is for heat or for cooling
									;short beep after toggle indicates heat, long beep, cooling

cpi		r16,4
brne	Decode5

rcall	ToggleWarmColdFlag

rjmp 	DecodeEnd


Decode5:						;toggle sonalert alarm on/off

cpi		r16,5
brne	Decode6					;Sonalert beep is heard when turned on
								;   two beeps when turned off

rcall	OnOffSonalertFlag				

rjmp 	DecodeEnd


Decode6:							;set switching temp manually

cpi		r16,6
brne	Decode7

rcall	SumR28R29					;sums plus one entries into r28,r29

rjmp	DecodeEnd


Decode7:

cpi		r16,7						;Set current temperature as switching temperature
brne	Decode8

rcall	RecordCurrentTempAsSwitchTemp

rjmp 	DecodeEnd


Decode8:					;Beep Out Current Temperature

cpi		r16,8
brne	DecodeEnd

rcall	ToggleBeepOutDataFlag

rjmp	DecodeEnd


DecodeEnd:					;all routines above end with a 3 second pause followed by a 1.5 seconds of 
							;   fast blinks
rcall	Delay3sec
ldi		r16,30
rcall	IntroBlink

ret


;---------------------------------

RecordCurrentTempAsSwitchTemp:					;Grabs current temp and uses it for new switching temp
									;   Data is permanently saved in EEPROM
lds		r16,$64
lds		r17,$65						;get low and high bytes of current temp

sts		$60,r16
sts		$61,r17						;store new switching temp value into SRAM

mov		r19,r16						;save r16, low byte current temp value into r19
mov		r20,r17						;save r17, high byte current temp value into r20

ldi		r16,$00						;address for switch temp value EEPROM save
mov		r17,r19						;  low byte

rcall	WriteEEPROM

ldi		r16,$01						;address for EEPROM save
mov		r17,r20						;  high byte

rcall	WriteEEPROM					;all data now written

rcall	iOneBeep

ret


;-------------------

PulseRelayOn:

sbr		r18,1		;set r18,0(1) - on relay flag

sbi		PortB,2
rcall	Delay025
cbi		PortB,2

ret


;----------

PulseRelayOff:

cbr		r18,1		;clr r18,0(1) - off relay flag

sbi		PortB,1
rcall	Delay025
cbi		PortB,1

ret


;-----------

ToggleRelay:

sbrs	r18,0		;skip if r18,0(1) is set
rjmp	ToggleRelay1

rcall	PulseRelayOff	;also clear r18,0(1) flag
rcall	iTwoBeeps


ret


ToggleRelay1:

rcall	PulseRelayOn 	;also set r18,0(1) flag
rcall	iOneBeep

ret


;----------

PulseRelayOnOff:		;Used to boot up relay in proper configuration with flag

rcall	PulseRelayOn

rcall	Delay100

rcall	PulseRelayOff

ret


;---------------

TwoBlinks:

sbi 	PortB,0			;on LED
rcall	Delay050
cbi		PortB,0
rcall	Delay500


OneBlink:

sbi 	PortB,0			;on LED
rcall	Delay050
cbi		PortB,0
rcall	Delay500

ret


;--------------------------------

VarBlink750:

push	r16


VarBlink750a:

sbi		PortB,0
rcall	Delay050
cbi		PortB,0
rcall	Delay750

dec		r16
brne	VarBlink750a

pop		r16

ret


;--------------------------

IntroBlink:			;slower blink rate designed for 1,000 type flashouts. 

						;  R16 has number of long blink sequences
push	r16
ldi		r16,15

IntroBlink1:

sbi		PortB,0
rcall	Delay025
cbi		PortB,0

rcall	Delay050
dec		r16
brne	IntroBlink1

pop		r16

ret


;-----------------------------

iIntroBlink:			;Interruptible IntroBlink for start of readout

push	r16

ldi		r16,50			;50 blinks


IIBa:

sbis	PIND,2			;skip if switch is up
rjmp	IIBend

sbi		PortB,0
rcall	iDelay050
cbi		PortB,0
rcall	iDelay050

dec		r16
brne	IIBa

rcall	iDelay2sec


IIBend:

pop		r16

ret


;--------------------------

;  Following Delay Routines include both interruptible and non-interruptible

IDelay025:			;1/40th second delay .025 seconds

push	r28
push	r29

sbis	PIND,2
rjmp	IDelay025end

ldi		r28,50
ldi		r29,130			;used FREEZE13.BAS to determine values


IDelay025a:

sbis	PIND,2			;skip if button is up
rjmp	IDelay025end

sbiw	r28,1			;subtract one from r28,29 pair
brne	IDelay025a


IDelay025end:

pop		r29
pop		r28

ret


;-----

IDelay100:			;Interruptible Delay of .1 second

push	r16

sbis	PIND,2
rjmp	IDelay100End

ldi		r16,4


IDelay100a:

sbis	PIND,2
rjmp	IDelay100End

rcall	IDelay025
dec		r16
brne	IDelay100a


IDelay100End:

pop		r16

ret


;---

iDelay500:

push	r16
ldi		r16,20

iDelay500a:

sbis	PIND,2		;skip if switch is ujp
rjmp	iDelay500end

rcall	iDelay025

dec		r16
brne	iDelay500a


iDelay500end:

pop		r16

ret


;----------------------------

iDelay750:

push	r16
ldi		r16,30		;3/4 second delay


iDelay750a:

sbis	PIND,2			;skip if switch is up
rjmp	iDelay750end

rcall	iDelay025
dec		r16
brne	iDelay750a


iDelay750end:

pop		r16

ret


;---

IDelay1sec:

push	r16

sbis	PIND,2
rjmp	IDelay1secEnd

ldi		r16,40

sbis	PIND,2
rjmp	IDelay1secEnd		;pop r16 and return


IDelay1sec1:

rcall	IDelay025
dec		r16
brne	IDelay1sec1


IDelay1secEnd:

pop		r16	
		
ret						;pop r16 and return


;-----

IDelay1MIN:			;Delay for one minute

push	r16

sbis	PIND,2
rjmp	IDelay1MINend

ldi		r16,60


IDelay1MINa:

sbis	PIND,2
rjmp	IDelay1MINend

rcall	IDelay1sec
dec		r16
brne	IDelay1MINa


IDelay1MINend:

pop		r16
ret


;-----

IVarMinDelay:		;r16 has minutes of delay

push	r16

sbis	PIND,2
rjmp	IVarMinDelayEnd


IVarMinDelay1:

rcall 	IDelay1MIN
dec		r16
brne	IVarMinDelay1


IVarMinDelayEnd:

pop		r16

ret


;---------------------------------

Delay025:			;non interruptible delay 1/40th second

push	r28
push	r29

ldi		r28,74
ldi		r29,195		;used FEEZE13.BAS for calculations


Delay025a:

sbiw	r28,1		;subtract one from count
brne	Delay025a


DelayEnd:

pop		r29
pop		r28

ret


;-----

Delay050:			;1/20th second delay

rcall	Delay025
rcall	Delay025

ret


;-----

iDelay050:			;1/20th second delay

rcall	iDelay025
rcall	iDelay025

ret


;----


Delay100:			;1/10th second delay

rcall	Delay050
rcall	Delay050

ret


;-----

Delay500:		;1/2 second delay

push	r16
ldi		r16,20


Delay500a:

rcall	Delay025
dec		r16
brne	Delay500a

pop		r16

ret


;-----

Delay750:

push	r16
ldi		r16,30		;3/4 second delay


Delay750a:

rcall	Delay025
dec		r16
brne	Delay750a

pop		r16

ret


;-----

Delay1sec:			;1 second delay

push	r16
ldi		r16,40


Delay1sec1:

rcall	Delay025
dec		r16
brne	Delay1sec1

pop		r16

ret


;-----

Delay2sec:

push	r16

ldi		r16,80


Delay2sec1:

rcall	Delay025
dec		r16
brne	Delay2sec1

pop		r16

ret


;-----

iDelay2sec:

push	r16

ldi		r16,80


iDelay2sec1:

sbis	PIND,2				;skip if switch is up
rjmp	iDelay2secEnd

rcall	iDelay025
dec		r16
brne	iDelay2sec1


iDelay2secEnd:

pop		r16

ret


;-----


Delay3sec:

push	r16
ldi		r16,120


Delay3sec1:

rcall	Delay025
dec		r16
brne	Delay3sec1

pop		r16

ret

;----

Delay1min:

push	r16
ldi		r16,60


Delay1min1:

rcall	Delay1sec
dec		r16
brne	Delay1min1

pop		r16

ret


;--------------------------

Var1MinDelay:	;(r16 has number of minutes delay -- 4.25 hours = 255)

push	r16


Var1MinDelay1:

rcall	Delay1min
dec		r16
brne	Var1MinDelay1

pop		r16

ret


;-----

VarSecDelay:	;r16 has number of seconds to delay --- up to 4.25 minutes

push	r16


VarSecDelay1:

rcall	Delay1sec
dec		r16
brne	VarSecDelay1

pop		r16

ret


;---------------------------------------

WriteEEPROM:	;r16 = address, r17 = data

sbic	EECR,EEPE
rjmp	WriteEEPROM

out		EEAR,r16 	;out address

out		EEDR,r17    ;out data
sbi		EECR,EEMPE  ;out one into EEMPE
sbi		EECR,EEPE

ret


;---------------------------

FetchEEPROM:  ;r16 = address,   r17 = data received


FetchEP:	;R16 data in, R19 data out


sbic	EECR,EEWE
rjmp	FetchEP

out		EEAR,r16
sbi		EECR,EERE

in		r17,EEDR

ret


;------------------------------------------

ToggleWarmColdFlag:

sbrs	r18,3					;skip if r18,3(8), warm (1)   cold (0) ---  flag is set
rjmp	TWCF1					;set r18,3(8) bit - (relay action normal = 1)

cbr		r18,8					;clear r18,3(8) bit - relay action reversed 
rcall	iTwoBeeps

ret


TWCF1:

sbr		r18,8					;relay action normal
rcall	iOneBeep

ret


;------------------------------------------------
;ADDITIONS - NOV. 25, 2013
;-------------------------------------------------

SumR28R29:		;Sums up total input 1,000, 100, 10, and 1 in four sequences
				;  R28, R29 have total count after values are input
				;R28,29 values stored in SRAM ($60,$61) and EEPROM ($01,$02)

ldi		r16,4				;Four blinks indicating 4 etries needed
rcall	VarBlink750

clr		r28
clr		r29

rcall	StallButtonPush

lds		r16,$7B		;get one less count from $7B for entry value
rcall	X1000		;thousands entry
rcall	Delay2sec

rcall	OneBlink	;entry accepted

;---

rcall	StallButtonPush

lds		r16,$7B		;get entry (one less)
rcall	X100		;hundreds entry
rcall	Delay2sec

rcall	OneBlink

;---

rcall	StallButtonPush

lds		r16,$7B
rcall	X10
rcall	Delay2sec

rcall	OneBlink

;---

rcall	StallButtonPush

lds		r16,$7B
rcall	X1
rcall	Delay2sec

;---

mov		r17,r28			;Save to EEPROM and SRAM
ldi		r16,$00			;EEPROM location
rcall	WriteEEPROM
mov		r17,r29
ldi		r16,$01
rcall	WriteEEPROM

sts		$60,r28
sts		$61,r29			;put in SRAM

ret


;------------------------------

X1000:					;Multiplies r16 entry by 1000 and puts in r28,29 pair

tst		r16
breq	X1000End


X1000a:

ldi		r17,20			;X20 for 1000 entry


X1000b:

adiw	r28,50			;add 50 to r28,29 pair
dec		r17
brne	X1000b


dec		r16
brne	X1000a


X1000End:

ret


;--------------------

X100:					;Entry X 100

tst		r16
breq	X100End


X100a:

ldi		r17,2


X100b:

adiw	r28,50			;Add 50 to r28,29
dec		r17
brne	X100b

dec		r16
brne	X100a


X100End:

ret


;---------------------

X10:					;multiply by ten

tst		r16
breq	X10End


X10b:

adiw	r28,10
dec		r16
brne	X10b


X10End:

ret


;------------------------

X1:						;add one to r28,29 word

tst		r16	
breq	X1End


X1b:

adiw	r28,1			;add one to word
dec		r16
brne	X1b


X1End:

ret


;-----------------------------

DAH:

sbi		PortB,0				;on LED
rcall	Delay750
cbi		PortB,0				;off LED

rcall	Delay750

ret


;---------------------------------

iVarDAH:

push	r16


iVarDAH1:

sbis	PIND,2				;skip if PIND,2 --- switch is up
rjmp	iVarDAHend

sbi		PortB,0				;on LED
rcall	iDelay500
cbi		PortB,0				;off LED

rcall	iDelay750

dec		r16
brne	iVarDAH1

rcall	iDelay2sec


iVarDAHend:

pop		r16

ret


;------------------------------------

DIT:

sbi		PortB,0
rcall	Delay050
cbi		PortB,0

rcall	Delay750

ret


;----------------------------------------

iVarDIT:

push	r16


iVarDIT1:

sbi		PortB,0
rcall	iDelay050
cbi		PortB,0

rcall	iDelay750

dec		r16
brne	iVarDIT1

rcall	iDelay2sec


iVarDITend:

pop		r16

ret


;---------------------------

iTwoSecBlink:		;blinks LED for one second - interruptible

sbis	PIND,2		;skip if swith is up
rjmp	OSBend

sbi		PortB,0		;ON led
rcall	iDelay2sec
cbi		PortB,0		;OFF led


OSBend:

ret


;-----------------------------

BeepSonalert:		;beep sonalertt 1/2 second 

sbi		PortB,7		;on Sonalert
rcall	Delay500
cbi		PortB,7		;off Sonalert

ret


;-----------------------------

iTwoBeeps:			;beeps sonalert twice

sbi		PortB,7
rcall	iDelay100
cbi		PortB,7

rcall	iDelay100


iOneBeep:			;beeps sonalert once

sbi		PortB,7
rcall	iDelay100
cbi		PortB,7

rcall	iDelay100

ret


;--------------------------

iVarBeep:			;r16 has numnber of beeps

push	r16


iVarBeep1:

sbis	PIND,2		;skip if PIND,2 ... switch is up
rjmp	iVarBeepEnd

sbi		PortB,7
rcall	iDelay100
cbi		PortB,7

rcall	iDelay750

dec		r16
brne	iVarBeep1

rcall	iDelay2sec


iVarBeepEnd:

pop		r16

ret


;------------------------


OnOffSonalertFlag:				;toggles sonalert beep on/off flag

sbrc	r18,6					;skip if r18,6(64) sonalert on flag = 1
rjmp	OnOffSonalert1

sbr		r18,64					;On solalert flag
rcall	iOneBeep					;one beep indicates that Sonalert is On
					
ret


OnOffSonalert1:

cbr		r18,64				;no beep if turned off
rcall	iTwoBeeps			;two beeps = sonalert is OFF

ret


;-------------------------------------

iVarBlink750:			;Blinks number of times in r16

Push	r16


iVarBlink1:

sbis	PIND,2			;skip if switch is up
rjmp	iVarBlinkEnd

sbi		PortB,0			;ON led
rcall	iDelay100		;delay 100 milliseconds
cbi		PortB,0			;OFF led

rcall	iDelay750		;delay between blinks  

dec		r16
brne	iVarBlink1

rcall	iDelay2sec		;delay after series of blinks


iVarBlinkEnd:

pop		r16

ret


;---------------------------

Stall:					;Stalls, waiting for push button.  When pushed, routine returns

rcall	Delay025
sbic	PIND,2			;skip if button is down
rjmp	Stall


Stall1:

rcall	Delay025
sbis	PIND,2
rjmp	Stall1

ret


;--------------------------------

DivideSub:				;(Dec. 28,2013 version) Takes r28,29 and divides them into 4 digits.
						;  each digit, 0-9 with MSD in r0, LSD in r3.
						;r16 is counter, r26,27 pair are temporary holding registers

push	r26
push	r27
push	r28
push	r29

;1000s subtraction

clr		r16				;zero R16


DSa:

mov		r26,r28
mov		r27,r29			;save in temp register for exit use


DSa1:

ldi		r17,20			;number of iterations counter


DSa2:					;1000 subtraction (20 X 50)

sbiw	r28,50
brcs	DSa3End
dec		r17
brne	DSa2

inc		r16				;increment 1000s counter
rjmp	DSa


DSa3End:				;save 1000s count in r0

mov		r28,r26
mov		r29,r27			;restore original save before subtraction

mov		r0,r16


;100 subtraction

clr		r16				;clear 100 count


DSb:

mov		r26,r28
mov		r27,r29


DSb1:

ldi		r17,2			;number of iterations


DSb2:

sbiw	r28,50
brcs	DSb3End
dec		r17
brne	DSb2

inc		r16				;increment 100s count
rjmp	DSb


DSb3End:				;save 100s count in r1

mov		r28,r26
mov		r29,r27			;restore original save before decrement

mov		r1,r16			;save 100 count in r1


;10s count

clr		r16				;10s count


DSc:

mov		r26,r28
mov		r27,r29			;save in temp stora for end


DSc1:

sbiw	r28,10			;subtract 10
brcs	DSc3End

inc		r16
rjmp	DSc


DSc3End:

mov		r28,r26
mov		r29,r27			;restore original save before decrement

mov		r2,r16			;save 10 count in r2


;UNITS count

mov		r3,r28			;remainder goes as units count to r3


pop		r29
pop		r28
pop		r27
pop		r26

ret


;Conversion to Digits Complete


;-----------------------------

iFinalDisplay:				;r28,29 pair have data to be displayed

sbrs	r18,6				;skip if r18,6(64) is set -- Sonalert warning flag is on
							;  which signals start of new interval disiplay

rjmp	iFD1				;continue process with no beeping


Beeps1:						;Beep twice if relay is on, once if relay is off

sbrs	r18,0				;skip if r18,0(1) is set (relay status is on)
rjmp	Beeps1a

rcall	iRaspBeep			;flows on to produce second beep			;


Beeps1a:					;relay status is off (1)

rcall	iRaspBeep			;give out one beep

							;continues on

;---


iFD1:

rcall	iDelay2sec
rcall	iDelay2sec			;allow 4 seconds before display starts

rcall	iIntroBlink

rcall	DivideSub			;separate r28,29 into individual 0-9 digits


iFD3:

rcall	iFlashOutTemp		;values in r0, r1, r2, r3

sbrs	r18,1				;skip if r18,1(2) - beep out data flag is set to OFF
rjmp	iFDend

rcall	iBeepOutTemp			;values in r0, r1, r2, r3


iFDend:

ret


;-----------------------

ToggleRelayWithEscape:		;toggles relay on and off, disconnecting from main loop.
							;  DIT pushes toggle relay.  DAH push exits to main loop
push	r16
rcall	PulseRelayOn


TRelay1:

sbic	PIND,2				;skip if switch is clear
rjmp	TRelay1				;jump back until switch is pushed  


TRelay2:

clr		r16
rcall	ToggleRelay


TRelay3:

rcall	Delay025
inc		r16
sbis	PIND,2				;skip if switch is up
rjmp	TRelay3


TRelay4:

cpi		r16,20	
brcs	TRelay1				;jump back to wait for DIT toggle command


TRelay5:					;jump out -- exit to Main Loop

pop		r16
ret


;---------------------------

iBeepOutSwitchTemp:

push		r28
push		r29

lds			r28,$60
lds			r29,$61

rjmp		iBeepOutSwitchTemp1	;continues this subroutine in iBeepOutTemp subroutine


;---------------------------

iBeepOutTemp:				;beeps out current temperature

push	r28
push	r29


iBeepOutSwitchTemp1:				;continues iBeepOutSwitchTemp subroutine call

rcall	DivideSub			;Put individual r0,1,2,3 values in place

sbis	PIND,2				;skip if switch is up
rjmp	iBOTend

rcall	iThreeBeepsWarning

rcall	iDelay2sec
rcall	iDelay1sec

mov		r16,r0
cpi		r16,0		
brne	iBOT1

rcall	iBOTdah
rjmp	iBOT1a


iBOT1:

rcall	iVarBeep


;---

iBOT1a:

rcall	iDelay2sec
rcall	iDelay1sec

mov		r16,r1
cpi		r16,0
brne	iBOT2

rcall	iBOTdah
rjmp	iBOT2a


iBOT2:

rcall	iVarBeep


;---

iBOT2a:

rcall	iDelay2sec
rcall	iDelay1sec

mov		r16,r2
cpi		r16,0
brne	iBOT3

rcall	iBOTdah
rjmp	iBOT3a


iBOT3:

rcall	iVarBeep


;---

iBOT3a:

rcall	iDelay2sec
rcall	iDelay1sec

mov		r16,r3
cpi		r16,0
brne	iBOT4

rcall	iBOTdah
rjmp	iBOT5


iBOT4:

rcall	iVarBeep


;---

iBOT5:

rcall	iDelay2sec
rcall	iDelay1sec


iBOTend:

pop		r29
pop		r28

ret


;-----------------------------

iThreeBeepsWarning:

push	r16

sbis	PIND,2			;skip if switch is up
rjmp	iTBWend

ldi		r16,3			;for three short beeps


iTBW1:

sbis	PIND,2			;skip if switch is up
rjmp	iTBWend

sbi		PortB,7
rcall	iDelay050
cbi		PortB,7
rcall	iDelay050
dec		r16
brne	iTBW1

rcall	iDelay2sec


iTBWend:

pop		r16

ret


;-----------------------------

iBOTdah:					;gives 2 second DAH

sbi		PortB,7			;ON sonalert
rcall	iDelay2sec
cbi		PortB,7			;OFF sonalert
rcall	iDelay750

ret


;------------------------------

iFlashOutTemp:				;beeps out current temperature

push	r28
push	r29

sbis	PIND,2				;skip if switch is up
rjmp	iFOTend

rcall	iDelay2sec
rcall	iDelay2sec

mov		r16,r0
cpi		r16,0		
brne	iFOT1

rcall	iFOTdah
rjmp	iFOT1a


iFOT1:

rcall	iVarBlink750

;---

iFOT1a:

rcall	iDelay2sec
rcall	iDelay2sec

mov		r16,r1
cpi		r16,0
brne	iFOT2

rcall	iFOTdah
rjmp	iFOT2a


iFOT2:

rcall	iVarBlink750


;---

iFOT2a:

rcall	iDelay2sec
rcall	iDelay2sec

mov		r16,r2
cpi		r16,0
brne	iFOT3

rcall	iFOTdah
rjmp	iFOT3a


iFOT3:

rcall	iVarBlink750


;---

iFOT3a:

rcall	iDelay2sec
rcall	iDelay2sec

mov		r16,r3
cpi		r16,0
brne	iFOT4

rcall	iFOTdah
rjmp	iFOT5


iFOT4:

rcall	iVarBlink750


;---

iFOT5:

rcall	iDelay2sec
rcall	iDelay2sec


iFOTend:

pop		r29
pop		r28

ret


;-----------------------------

iFOTdah:					;gives 2 second DAH

sbi		PortB,0			;ON sonalert
rcall	iDelay2sec
cbi		PortB,0			;OFF sonalert
rcall	iDelay750

ret

;------------------------------

ToggleBeepOutDataFlag:

rcall	Delay500

sbrs	r18,1			;skip if r18,1(2) Beep Out Data flag is set
rjmp	TOBD1

cbr		r18,2			;clear Beep Out Data flag
rcall	iTwoBeeps

ret


TOBD1:					;set r18,1(2) Beep Out Data flag

sbr		r18,2			;on Beep Out Data flag
rcall	iOneBeep

ret


;------------------------------

iRaspBeep:

push	r16
push	r17

ldi		r17,4


RB1:

sbis	PIND,2			;skip if switch is up
rjmp	RBend

sbi		PortB,7			;on sonalert
rcall	iDelay025
cbi		PortB,7			;off sonalert

rcall	iDelay025
dec		r17
brne	RB1

rcall	iDelay500		;1/2 second delay 


RBend:

pop		r17
pop		r16

ret


;--------------------------------
